@@ -2,15 +2,17 @@
 # Copyright (c) 2017 Derek Gustafson <degustaf@gmail.com>
 # Copyright (c) 2019, 2021 Pierre Sassoulas <pierre.sassoulas@gmail.com>
 # Copyright (c) 2020 hippo91 <guillaume.peillex@gmail.com>
+# Copyright (c) 2021 Marc Mueller <30130371+cdce8p@users.noreply.github.com>
 
 # Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
-# For details: https://github.com/PyCQA/pylint/blob/master/LICENSE
+# For details: https://github.com/PyCQA/pylint/blob/main/LICENSE
 
 """Checker for anything related to the async protocol (PEP 492)."""
 
 import sys
 
 import astroid
+from astroid import nodes
 
 from pylint import checkers, interfaces, utils
 from pylint.checkers import utils as checker_utils
@@ -45,9 +47,9 @@ class AsyncChecker(checkers.BaseChecker):
 
     @checker_utils.check_messages("yield-inside-async-function")
     def visit_asyncfunctiondef(self, node):
-        for child in node.nodes_of_class(astroid.Yield):
+        for child in node.nodes_of_class(nodes.Yield):
             if child.scope() is node and (
-                sys.version_info[:2] == (3, 5) or isinstance(child, astroid.YieldFrom)
+                sys.version_info[:2] == (3, 5) or isinstance(child, nodes.YieldFrom)
             ):
                 self.add_message("yield-inside-async-function", node=child)
 
@@ -58,7 +60,7 @@ class AsyncChecker(checkers.BaseChecker):
             if inferred is None or inferred is astroid.Uninferable:
                 continue
 
-            if isinstance(inferred, astroid.AsyncFunctionDef):
+            if isinstance(inferred, nodes.AsyncFunctionDef):
                 # Check if we are dealing with a function decorated
                 # with contextlib.asynccontextmanager.
                 if decorated_with(inferred, self._async_generators):
