@@ -26,31 +26,55 @@
 # Copyright (c) 2020 hippo91 <guillaume.peillex@gmail.com>
 # Copyright (c) 2020 谭九鼎 <109224573@qq.com>
 # Copyright (c) 2020 Anthony Sottile <asottile@umich.edu>
+# Copyright (c) 2021 Marc Mueller <30130371+cdce8p@users.noreply.github.com>
+# Copyright (c) 2021 Daniël van Noord <13665637+DanielNoord@users.noreply.github.com>
+# Copyright (c) 2021 Yilei "Dolee" Yang <yileiyang@google.com>
 # Copyright (c) 2021 Matus Valo <matusvalo@users.noreply.github.com>
+# Copyright (c) 2021 victor <16359131+jiajunsu@users.noreply.github.com>
 
 # Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
-# For details: https://github.com/PyCQA/pylint/blob/master/LICENSE
+# For details: https://github.com/PyCQA/pylint/blob/main/LICENSE
 
 """Checkers for various standard library functions."""
 
 import sys
+from collections.abc import Iterable
 
 import astroid
+from astroid import nodes
 
 from pylint.checkers import BaseChecker, DeprecatedMixin, utils
 from pylint.interfaces import IAstroidChecker
 
-OPEN_FILES = {"open", "file"}
+OPEN_FILES_MODE = ("open", "file")
+OPEN_FILES_ENCODING = ("open",)
 UNITTEST_CASE = "unittest.case"
 THREADING_THREAD = "threading.Thread"
 COPY_COPY = "copy.copy"
 OS_ENVIRON = "os._Environ"
-ENV_GETTERS = {"os.getenv"}
+ENV_GETTERS = ("os.getenv",)
 SUBPROCESS_POPEN = "subprocess.Popen"
 SUBPROCESS_RUN = "subprocess.run"
 OPEN_MODULE = "_io"
+DEBUG_BREAKPOINTS = ("builtins.breakpoint", "sys.breakpointhook", "pdb.set_trace")
+
+
+DEPRECATED_MODULES = {
+    (0, 0, 0): {"tkinter.tix", "fpectl"},
+    (3, 2, 0): {"optparse"},
+    (3, 4, 0): {"imp"},
+    (3, 5, 0): {"formatter"},
+    (3, 6, 0): {"asynchat", "asyncore"},
+    (3, 7, 0): {"macpath"},
+    (3, 9, 0): {"lib2to3", "parser", "symbol", "binhex"},
+}
 
 DEPRECATED_ARGUMENTS = {
+    (0, 0, 0): {
+        "int": ((None, "x"),),
+        "bool": ((None, "x"),),
+        "float": ((None, "x"),),
+    },
     (3, 8, 0): {
         "asyncio.tasks.sleep": ((None, "loop"),),
         "asyncio.tasks.gather": ((None, "loop"),),
@@ -62,6 +86,8 @@ DEPRECATED_ARGUMENTS = {
         "asyncio.subprocess.create_subprocess_shell": ((4, "loop"),),
         "gettext.translation": ((5, "codeset"),),
         "gettext.install": ((2, "codeset"),),
+        "functools.partialmethod": ((None, "func"),),
+        "weakref.finalize": ((None, "func"), (None, "obj")),
         "profile.Profile.runcall": ((None, "func"),),
         "cProfile.Profile.runcall": ((None, "func"),),
         "bdb.Bdb.runcall": ((None, "func"),),
@@ -81,6 +107,15 @@ DEPRECATED_ARGUMENTS = {
     (3, 9, 0): {"random.Random.shuffle": ((1, "random"),)},
 }
 
+DEPRECATED_DECORATORS = {
+    (3, 8, 0): {"asyncio.coroutine"},
+    (3, 3, 0): {
+        "abc.abstractclassmethod",
+        "abc.abstractstaticmethod",
+        "abc.abstractproperty",
+    },
+}
+
 
 DEPRECATED_METHODS = {
     0: {
@@ -136,6 +171,7 @@ DEPRECATED_METHODS = {
             "base64.decodestring",
             "ntpath.splitunc",
             "os.path.splitunc",
+            "os.stat_float_times",
         },
         (3, 2, 0): {
             "cgi.escape",
@@ -152,6 +188,8 @@ DEPRECATED_METHODS = {
             "logging.LoggerAdapter.warn",
             "nntplib._NNTPBase.xpath",
             "platform.popen",
+            "sqlite3.OptimizedUnicode",
+            "time.clock",
         },
         (3, 4, 0): {
             "importlib.find_loader",
@@ -168,15 +206,20 @@ DEPRECATED_METHODS = {
             "platform.linux_distribution",
             "platform.dist",
         },
-        (3, 6, 0): {"importlib._bootstrap_external.FileLoader.load_module"},
+        (3, 6, 0): {
+            "importlib._bootstrap_external.FileLoader.load_module",
+            "_ssl.RAND_pseudo_bytes",
+        },
         (3, 7, 0): {
             "sys.set_coroutine_wrapper",
             "sys.get_coroutine_wrapper",
             "aifc.openfp",
+            "threading.Thread.isAlive",
             "asyncio.Task.current_task",
             "asyncio.Task.all_task",
             "locale.format",
             "ssl.wrap_socket",
+            "ssl.match_hostname",
             "sunau.openfp",
             "wave.openfp",
         },
@@ -196,6 +239,67 @@ DEPRECATED_METHODS = {
             "binascii.rlecode_hqx",
             "binascii.rledecode_hqx",
         },
+        (3, 10, 0): {
+            "_sqlite3.enable_shared_cache",
+            "pathlib.Path.link_to",
+            "zipimport.zipimporter.load_module",
+            "zipimport.zipimporter.find_module",
+            "zipimport.zipimporter.find_loader",
+            "threading.currentThread",
+            "threading.activeCount",
+            "threading.Condition.notifyAll",
+            "threading.Event.isSet",
+            "threading.Thread.setName",
+            "threading.Thread.getName",
+            "threading.Thread.isDaemon",
+            "threading.Thread.setDaemon",
+            "cgi.log",
+        },
+    },
+}
+
+
+DEPRECATED_CLASSES = {
+    (3, 3, 0): {
+        "importlib.abc": {
+            "Finder",
+        },
+        "pkgutil": {
+            "ImpImporter",
+            "ImpLoader",
+        },
+        "collections": {
+            "Awaitable",
+            "Coroutine",
+            "AsyncIterable",
+            "AsyncIterator",
+            "AsyncGenerator",
+            "Hashable",
+            "Iterable",
+            "Iterator",
+            "Generator",
+            "Reversible",
+            "Sized",
+            "Container",
+            "Callable",
+            "Collection",
+            "Set",
+            "MutableSet",
+            "Mapping",
+            "MutableMapping",
+            "MappingView",
+            "KeysView",
+            "ItemsView",
+            "ValuesView",
+            "Sequence",
+            "MutableSequence",
+            "ByteString",
+        },
+    },
+    (3, 9, 0): {
+        "smtpd": {
+            "MailmanProxy",
+        }
     },
 }
 
@@ -315,6 +419,29 @@ class StdlibChecker(DeprecatedMixin, BaseChecker):
             "deprecated-argument",
             "The argument is marked as deprecated and will be removed in the future.",
         ),
+        "W1512": (
+            "Using deprecated class %s of module %s",
+            "deprecated-class",
+            "The class is marked as deprecated and will be removed in the future.",
+        ),
+        "W1513": (
+            "Using deprecated decorator %s()",
+            "deprecated-decorator",
+            "The decorator is marked as deprecated and will be removed in the future.",
+        ),
+        "W1514": (
+            "Using open without explicitly specifying an encoding",
+            "unspecified-encoding",
+            "It is better to specify an encoding when opening documents. "
+            "Using the system default implicitly can create problems on other operating systems. "
+            "See https://www.python.org/dev/peps/pep-0597/",
+        ),
+        "W1515": (
+            "Leaving functions creating breakpoints in production code is not recommended",
+            "forgotten-debug-statement",
+            "Calls to breakpoint(), sys.breakpointhook() and pdb.set_trace() should be removed "
+            "from code that is not actively being debugged.",
+        ),
     }
 
     def __init__(self, linter=None):
@@ -324,10 +451,22 @@ class StdlibChecker(DeprecatedMixin, BaseChecker):
         for since_vers, func_list in DEPRECATED_METHODS[sys.version_info[0]].items():
             if since_vers <= sys.version_info:
                 self._deprecated_methods.update(func_list)
-        self._deprecated_attributes = dict()
+        self._deprecated_attributes = {}
         for since_vers, func_list in DEPRECATED_ARGUMENTS.items():
             if since_vers <= sys.version_info:
                 self._deprecated_attributes.update(func_list)
+        self._deprecated_classes = {}
+        for since_vers, class_list in DEPRECATED_CLASSES.items():
+            if since_vers <= sys.version_info:
+                self._deprecated_classes.update(class_list)
+        self._deprecated_modules = set()
+        for since_vers, mod_list in DEPRECATED_MODULES.items():
+            if since_vers <= sys.version_info:
+                self._deprecated_modules.update(mod_list)
+        self._deprecated_decorators = set()
+        for since_vers, decorator_list in DEPRECATED_DECORATORS.items():
+            if since_vers <= sys.version_info:
+                self._deprecated_decorators.update(decorator_list)
 
     def _check_bad_thread_instantiation(self, node):
         if not node.kwargs and not node.keywords and len(node.args) <= 1:
@@ -344,9 +483,13 @@ class StdlibChecker(DeprecatedMixin, BaseChecker):
         if "check" not in kwargs:
             self.add_message("subprocess-run-check", node=node)
 
-    def _check_shallow_copy_environ(self, node):
+    def _check_shallow_copy_environ(self, node: nodes.Call) -> None:
         arg = utils.get_argument_from_call(node, position=0)
-        for inferred in arg.inferred():
+        try:
+            inferred_args = arg.inferred()
+        except astroid.InferenceError:
+            return
+        for inferred in inferred_args:
             if inferred.qname() == OS_ENVIRON:
                 self.add_message("shallow-copy-environ", node=node)
                 break
@@ -362,34 +505,47 @@ class StdlibChecker(DeprecatedMixin, BaseChecker):
         "invalid-envvar-default",
         "subprocess-popen-preexec-fn",
         "subprocess-run-check",
+        "deprecated-class",
+        "unspecified-encoding",
+        "forgotten-debug-statement",
     )
-    def visit_call(self, node):
+    def visit_call(self, node: nodes.Call) -> None:
         """Visit a Call node."""
-        try:
-            for inferred in node.func.infer():
-                if inferred is astroid.Uninferable:
-                    continue
-                if inferred.root().name == OPEN_MODULE:
-                    if getattr(node.func, "name", None) in OPEN_FILES:
-                        self._check_open_mode(node)
-                elif inferred.root().name == UNITTEST_CASE:
-                    self._check_redundant_assert(node, inferred)
-                elif isinstance(inferred, astroid.ClassDef):
-                    if inferred.qname() == THREADING_THREAD:
-                        self._check_bad_thread_instantiation(node)
-                    elif inferred.qname() == SUBPROCESS_POPEN:
-                        self._check_for_preexec_fn_in_popen(node)
-                elif isinstance(inferred, astroid.FunctionDef):
-                    name = inferred.qname()
-                    if name == COPY_COPY:
-                        self._check_shallow_copy_environ(node)
-                    elif name in ENV_GETTERS:
-                        self._check_env_function(node, inferred)
-                    elif name == SUBPROCESS_RUN:
-                        self._check_for_check_kw_in_run(node)
-                self.check_deprecated_method(node, inferred)
-        except astroid.InferenceError:
-            return
+        self.check_deprecated_class_in_call(node)
+        for inferred in utils.infer_all(node.func):
+            if inferred is astroid.Uninferable:
+                continue
+            if inferred.root().name == OPEN_MODULE:
+                if (
+                    isinstance(node.func, nodes.Name)
+                    and node.func.name in OPEN_FILES_MODE
+                ):
+                    self._check_open_mode(node)
+                if (
+                    isinstance(node.func, nodes.Name)
+                    and node.func.name in OPEN_FILES_ENCODING
+                    or isinstance(node.func, nodes.Attribute)
+                    and node.func.attrname in OPEN_FILES_ENCODING
+                ):
+                    self._check_open_encoded(node)
+            elif inferred.root().name == UNITTEST_CASE:
+                self._check_redundant_assert(node, inferred)
+            elif isinstance(inferred, nodes.ClassDef):
+                if inferred.qname() == THREADING_THREAD:
+                    self._check_bad_thread_instantiation(node)
+                elif inferred.qname() == SUBPROCESS_POPEN:
+                    self._check_for_preexec_fn_in_popen(node)
+            elif isinstance(inferred, nodes.FunctionDef):
+                name = inferred.qname()
+                if name == COPY_COPY:
+                    self._check_shallow_copy_environ(node)
+                elif name in ENV_GETTERS:
+                    self._check_env_function(node, inferred)
+                elif name == SUBPROCESS_RUN:
+                    self._check_for_check_kw_in_run(node)
+                elif name in DEBUG_BREAKPOINTS:
+                    self.add_message("forgotten-debug-statement", node=node)
+            self.check_deprecated_method(node, inferred)
 
     @utils.check_messages("boolean-datetime")
     def visit_unaryop(self, node):
@@ -413,7 +569,7 @@ class StdlibChecker(DeprecatedMixin, BaseChecker):
         if (
             isinstance(infer, astroid.BoundMethod)
             and node.args
-            and isinstance(node.args[0], astroid.Const)
+            and isinstance(node.args[0], nodes.Const)
             and infer.name in ["assertTrue", "assertFalse"]
         ):
             self.add_message(
@@ -444,11 +600,36 @@ class StdlibChecker(DeprecatedMixin, BaseChecker):
             return
         if mode_arg:
             mode_arg = utils.safe_infer(mode_arg)
-            if isinstance(mode_arg, astroid.Const) and not _check_mode_str(
+            if isinstance(mode_arg, nodes.Const) and not _check_mode_str(
                 mode_arg.value
             ):
                 self.add_message("bad-open-mode", node=node, args=mode_arg.value)
 
+    def _check_open_encoded(self, node: nodes.Call) -> None:
+        """Check that the encoded argument of an open call is valid."""
+        mode_arg = None
+        try:
+            mode_arg = utils.get_argument_from_call(node, position=1, keyword="mode")
+        except utils.NoSuchArgumentError:
+            pass
+
+        if mode_arg:
+            mode_arg = utils.safe_infer(mode_arg)
+        if not mode_arg or "b" not in mode_arg.value:
+            encoding_arg = None
+            try:
+                encoding_arg = utils.get_argument_from_call(
+                    node, position=None, keyword="encoding"
+                )
+            except utils.NoSuchArgumentError:
+                self.add_message("unspecified-encoding", node=node)
+
+            if encoding_arg:
+                encoding_arg = utils.safe_infer(encoding_arg)
+
+                if isinstance(encoding_arg, nodes.Const) and encoding_arg.value is None:
+                    self.add_message("unspecified-encoding", node=node)
+
     def _check_env_function(self, node, infer):
         env_name_kwarg = "key"
         env_value_kwarg = "default"
@@ -493,7 +674,7 @@ class StdlibChecker(DeprecatedMixin, BaseChecker):
             return
 
         name = infer.qname()
-        if isinstance(call_arg, astroid.Const):
+        if isinstance(call_arg, nodes.Const):
             emit = False
             if call_arg.value is None:
                 emit = not allow_none
@@ -504,13 +685,23 @@ class StdlibChecker(DeprecatedMixin, BaseChecker):
         else:
             self.add_message(message, node=node, args=(name, call_arg.pytype()))
 
+    def deprecated_modules(self):
+        """Callback returning the deprecated modules."""
+        return self._deprecated_modules
+
     def deprecated_methods(self):
         return self._deprecated_methods
 
     def deprecated_arguments(self, method: str):
         return self._deprecated_attributes.get(method, ())
 
+    def deprecated_classes(self, module: str):
+        return self._deprecated_classes.get(module, ())
+
+    def deprecated_decorators(self) -> Iterable:
+        return self._deprecated_decorators
+
 
 def register(linter):
-    """required method to auto register this checker """
+    """required method to auto register this checker"""
     linter.register_checker(StdlibChecker(linter))
