@@ -24,27 +24,33 @@
 # Copyright (c) 2020 hippo91 <guillaume.peillex@gmail.com>
 # Copyright (c) 2020 谭九鼎 <109224573@qq.com>
 # Copyright (c) 2020 Anthony <tanant@users.noreply.github.com>
+# Copyright (c) 2021 Marc Mueller <30130371+cdce8p@users.noreply.github.com>
+# Copyright (c) 2021 Daniël van Noord <13665637+DanielNoord@users.noreply.github.com>
+# Copyright (c) 2021 Peter Kolbus <peter.kolbus@garmin.com>
 
 
 # Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
-# For details: https://github.com/PyCQA/pylint/blob/master/LICENSE
+# For details: https://github.com/PyCQA/pylint/blob/main/LICENSE
 
 """Checker for string formatting operations.
 """
 
-import builtins
 import collections
 import numbers
 import re
 import tokenize
-from typing import Counter, Iterable
+from typing import TYPE_CHECKING, Iterable
 
 import astroid
+from astroid import nodes
 
 from pylint.checkers import BaseChecker, BaseTokenChecker, utils
 from pylint.checkers.utils import check_messages
 from pylint.interfaces import IAstroidChecker, IRawChecker, ITokenChecker
 
+if TYPE_CHECKING:
+    from typing import Counter  # typing.Counter added in Python 3.6.1
+
 _AST_NODE_STR_TYPES = ("__builtin__.unicode", "__builtin__.str", "builtins.str")
 # Prefixes for both strings and bytes literals per
 # https://docs.python.org/3/reference/lexical_analysis.html#string-and-bytes-literals
@@ -78,7 +84,7 @@ SINGLE_QUOTED_REGEX = re.compile("(%s)?'''" % "|".join(_PREFIXES))
 DOUBLE_QUOTED_REGEX = re.compile('(%s)?"""' % "|".join(_PREFIXES))
 QUOTE_DELIMITER_REGEX = re.compile("(%s)?(\"|')" % "|".join(_PREFIXES), re.DOTALL)
 
-MSGS = {
+MSGS = {  # pylint: disable=consider-using-namedtuple-or-dataclass
     "E1300": (
         "Unsupported format character %r (%#02x) at index %d",
         "bad-format-character",
@@ -199,22 +205,24 @@ MSGS = {
         "Used when we detect an f-string that does not use any interpolation variables, "
         "in which case it can be either a normal string or a bug in the code.",
     ),
+    "W1310": (
+        "Using formatting for a string that does not have any interpolated variables",
+        "format-string-without-interpolation",
+        "Used when we detect a string that does not have any interpolation variables, "
+        "in which case it can be either a normal string without formatting or a bug in the code.",
+    ),
 }
 
 OTHER_NODES = (
-    astroid.Const,
-    astroid.List,
-    astroid.Lambda,
-    astroid.FunctionDef,
-    astroid.ListComp,
-    astroid.SetComp,
-    astroid.GeneratorExp,
+    nodes.Const,
+    nodes.List,
+    nodes.Lambda,
+    nodes.FunctionDef,
+    nodes.ListComp,
+    nodes.SetComp,
+    nodes.GeneratorExp,
 )
 
-BUILTINS_STR = builtins.__name__ + ".str"
-BUILTINS_FLOAT = builtins.__name__ + ".float"
-BUILTINS_INT = builtins.__name__ + ".int"
-
 
 def get_access_path(key, parts):
     """Given a list of format specifiers, returns
@@ -235,11 +243,11 @@ def arg_matches_format_type(arg_type, format_type):
         return True
     if isinstance(arg_type, astroid.Instance):
         arg_type = arg_type.pytype()
-        if arg_type == BUILTINS_STR:
+        if arg_type == "builtins.str":
             return format_type == "c"
-        if arg_type == BUILTINS_FLOAT:
+        if arg_type == "builtins.float":
             return format_type in "deEfFgGn%"
-        if arg_type == BUILTINS_INT:
+        if arg_type == "builtins.int":
             # Integers allow all types
             return True
         return False
@@ -268,6 +276,7 @@ class StringFormatChecker(BaseChecker):
         "too-many-format-args",
         "too-few-format-args",
         "bad-string-format-type",
+        "format-string-without-interpolation",
     )
     def visit_binop(self, node):
         if node.op != "%":
@@ -275,7 +284,7 @@ class StringFormatChecker(BaseChecker):
         left = node.left
         args = node.right
 
-        if not (isinstance(left, astroid.Const) and isinstance(left.value, str)):
+        if not (isinstance(left, nodes.Const) and isinstance(left.value, str)):
             return
         format_string = left.value
         try:
@@ -296,6 +305,9 @@ class StringFormatChecker(BaseChecker):
         except utils.IncompleteFormatString:
             self.add_message("truncated-format-string", node=node)
             return
+        if not required_keys and not required_num_args:
+            self.add_message("format-string-without-interpolation", node=node)
+            return
         if required_keys and required_num_args:
             # The format string uses both named and unnamed format
             # specifiers.
@@ -305,11 +317,11 @@ class StringFormatChecker(BaseChecker):
             # Check that the RHS of the % operator is a mapping object
             # that contains precisely the set of keys required by the
             # format string.
-            if isinstance(args, astroid.Dict):
+            if isinstance(args, nodes.Dict):
                 keys = set()
                 unknown_keys = False
                 for k, _ in args.items:
-                    if isinstance(k, astroid.Const):
+                    if isinstance(k, nodes.Const):
                         key = k.value
                         if isinstance(key, str):
                             keys.add(key)
@@ -335,7 +347,7 @@ class StringFormatChecker(BaseChecker):
                             "unused-format-string-key", node=node, args=key
                         )
                 for key, arg in args.items:
-                    if not isinstance(key, astroid.Const):
+                    if not isinstance(key, nodes.Const):
                         continue
                     format_type = required_key_types.get(key.value, None)
                     arg_type = utils.safe_infer(arg)
@@ -349,7 +361,7 @@ class StringFormatChecker(BaseChecker):
                             node=node,
                             args=(arg_type.pytype(), format_type),
                         )
-            elif isinstance(args, (OTHER_NODES, astroid.Tuple)):
+            elif isinstance(args, (OTHER_NODES, nodes.Tuple)):
                 type_name = type(args).__name__
                 self.add_message("format-needs-mapping", node=node, args=type_name)
             # else:
@@ -362,13 +374,13 @@ class StringFormatChecker(BaseChecker):
             # the % operator matches the number required by the format
             # string.
             args_elts = ()
-            if isinstance(args, astroid.Tuple):
+            if isinstance(args, nodes.Tuple):
                 rhs_tuple = utils.safe_infer(args)
                 num_args = None
                 if hasattr(rhs_tuple, "elts"):
                     args_elts = rhs_tuple.elts
                     num_args = len(args_elts)
-            elif isinstance(args, (OTHER_NODES, (astroid.Dict, astroid.DictComp))):
+            elif isinstance(args, (OTHER_NODES, (nodes.Dict, nodes.DictComp))):
                 args_elts = [args]
                 num_args = 1
             else:
@@ -401,10 +413,10 @@ class StringFormatChecker(BaseChecker):
 
     @check_messages("f-string-without-interpolation")
     def visit_joinedstr(self, node):
-        if isinstance(node.parent, astroid.FormattedValue):
+        if isinstance(node.parent, nodes.FormattedValue):
             return
         for value in node.values:
-            if isinstance(value, astroid.FormattedValue):
+            if isinstance(value, nodes.FormattedValue):
                 return
         self.add_message("f-string-without-interpolation", node=node)
 
@@ -418,7 +430,7 @@ class StringFormatChecker(BaseChecker):
         ):
             if func.name in ("strip", "lstrip", "rstrip") and node.args:
                 arg = utils.safe_infer(node.args[0])
-                if not isinstance(arg, astroid.Const) or not isinstance(arg.value, str):
+                if not isinstance(arg, nodes.Const) or not isinstance(arg.value, str):
                     return
                 if len(arg.value) != len(set(arg.value)):
                     self.add_message(
@@ -431,7 +443,7 @@ class StringFormatChecker(BaseChecker):
 
     def _detect_vacuous_formatting(self, node, positional_arguments):
         counter = collections.Counter(
-            arg.name for arg in positional_arguments if isinstance(arg, astroid.Name)
+            arg.name for arg in positional_arguments if isinstance(arg, nodes.Name)
         )
         for name, count in counter.items():
             if count == 1:
@@ -441,7 +453,7 @@ class StringFormatChecker(BaseChecker):
             )
 
     def _check_new_format(self, node, func):
-        """Check the new string formatting. """
+        """Check the new string formatting."""
         # Skip format nodes which don't have an explicit string on the
         # left side of the format operation.
         # We do this because our inference engine can't properly handle
@@ -451,8 +463,8 @@ class StringFormatChecker(BaseChecker):
         #
         #    fmt = 'some string {}'.format
         #    fmt('arg')
-        if isinstance(node.func, astroid.Attribute) and not isinstance(
-            node.func.expr, astroid.Const
+        if isinstance(node.func, nodes.Attribute) and not isinstance(
+            node.func.expr, nodes.Const
         ):
             return
         if node.starargs or node.kwargs:
@@ -461,7 +473,7 @@ class StringFormatChecker(BaseChecker):
             strnode = next(func.bound.infer())
         except astroid.InferenceError:
             return
-        if not (isinstance(strnode, astroid.Const) and isinstance(strnode.value, str)):
+        if not (isinstance(strnode, nodes.Const) and isinstance(strnode.value, str)):
             return
         try:
             call_site = astroid.arguments.CallSite.from_call(node)
@@ -513,6 +525,9 @@ class StringFormatChecker(BaseChecker):
         if check_args:
             # num_args can be 0 if manual_pos is not.
             num_args = num_args or manual_pos
+            if not num_args:
+                self.add_message("format-string-without-interpolation", node=node)
+                return
             if len(positional_arguments) > num_args:
                 self.add_message("too-many-format-args", node=node)
             elif len(positional_arguments) < num_args:
@@ -553,7 +568,7 @@ class StringFormatChecker(BaseChecker):
                 # No need to check this key if it doesn't
                 # use attribute / item access
                 continue
-            if argument.parent and isinstance(argument.parent, astroid.Arguments):
+            if argument.parent and isinstance(argument.parent, nodes.Arguments):
                 # Ignore any object coming from an argument,
                 # because we can't infer its value properly.
                 continue
@@ -584,7 +599,7 @@ class StringFormatChecker(BaseChecker):
                     warn_error = False
                     if hasattr(previous, "getitem"):
                         try:
-                            previous = previous.getitem(astroid.Const(specifier))
+                            previous = previous.getitem(nodes.Const(specifier))
                         except (
                             astroid.AstroidIndexError,
                             astroid.AstroidTypeError,
@@ -651,6 +666,13 @@ class StringConstantChecker(BaseTokenChecker):
             "Quote delimiters are not used consistently throughout a module "
             "(with allowances made for avoiding unnecessary escaping).",
         ),
+        "W1406": (
+            "The u prefix for strings is no longer necessary in Python >=3.0",
+            "redundant-u-string-prefix",
+            "Used when we detect a string with a u prefix. These prefixes were necessary "
+            "in Python 2 to indicate a string was Unicode, but since Python 3.0 strings "
+            "are Unicode by default.",
+        ),
     }
     options = (
         (
@@ -734,7 +756,7 @@ class StringConstantChecker(BaseTokenChecker):
         self.check_for_concatenated_strings(node.elts, "tuple")
 
     def visit_assign(self, node):
-        if isinstance(node.value, astroid.Const) and isinstance(node.value.value, str):
+        if isinstance(node.value, nodes.Const) and isinstance(node.value.value, str):
             self.check_for_concatenated_strings([node.value], "assignment")
 
     def check_for_consistent_string_delimiters(
@@ -749,7 +771,8 @@ class StringConstantChecker(BaseTokenChecker):
         Args:
           tokens: The tokens to be checked against for consistent usage.
         """
-        string_delimiters: Counter[str] = collections.Counter()
+        # typing.Counter added in Python 3.6.1 so this type hint must be a comment
+        string_delimiters = collections.Counter()  # type: Counter[str]
 
         # First, figure out which quote character predominates in the module
         for tok_type, token, _, _, _ in tokens:
@@ -774,7 +797,7 @@ class StringConstantChecker(BaseTokenChecker):
     def check_for_concatenated_strings(self, elements, iterable_type):
         for elt in elements:
             if not (
-                isinstance(elt, astroid.Const) and elt.pytype() in _AST_NODE_STR_TYPES
+                isinstance(elt, nodes.Const) and elt.pytype() in _AST_NODE_STR_TYPES
             ):
                 continue
             if elt.col_offset < 0:
@@ -886,9 +909,25 @@ class StringConstantChecker(BaseTokenChecker):
             # character can never be the start of a new backslash escape.
             index += 2
 
+    @check_messages("redundant-u-string-prefix")
+    def visit_const(self, node: nodes.Const):
+        if node.pytype() == "builtins.str" and not isinstance(
+            node.parent, nodes.JoinedStr
+        ):
+            self._detect_u_string_prefix(node)
+
+    def _detect_u_string_prefix(self, node: nodes.Const):
+        """Check whether strings include a 'u' prefix like u'String'"""
+        if node.kind == "u":
+            self.add_message(
+                "redundant-u-string-prefix",
+                line=node.lineno,
+                col_offset=node.col_offset,
+            )
+
 
 def register(linter):
-    """required method to auto register this checker """
+    """required method to auto register this checker"""
     linter.register_checker(StringFormatChecker(linter))
     linter.register_checker(StringConstantChecker(linter))
 
