@@ -1,5 +1,5 @@
+# Copyright (c) 2016-2019, 2021 Ashley Whetter <ashley@awhetter.co.uk>
 # Copyright (c) 2016-2020 Claudiu Popa <pcmanticore@gmail.com>
-# Copyright (c) 2016-2019 Ashley Whetter <ashley@awhetter.co.uk>
 # Copyright (c) 2016 Yuri Bochkarev <baltazar.bz@gmail.com>
 # Copyright (c) 2016 Glenn Matthews <glenn@e-dad.net>
 # Copyright (c) 2016 Moises Lopez <moylop260@vauxoo.com>
@@ -14,9 +14,10 @@
 # Copyright (c) 2019 Danny Hermes <daniel.j.hermes@gmail.com>
 # Copyright (c) 2019 Zeb Nicholls <zebedee.nicholls@climate-energy-college.org>
 # Copyright (c) 2021 Pierre Sassoulas <pierre.sassoulas@gmail.com>
+# Copyright (c) 2021 Marc Mueller <30130371+cdce8p@users.noreply.github.com>
 
 # Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
-# For details: https://github.com/PyCQA/pylint/blob/master/LICENSE
+# For details: https://github.com/PyCQA/pylint/blob/main/LICENSE
 
 """Utility methods for docstring checking."""
 
@@ -24,6 +25,7 @@ import re
 from typing import List
 
 import astroid
+from astroid import nodes
 
 from pylint.checkers import utils
 
@@ -52,9 +54,9 @@ def get_setters_property_name(node):
     decorators = node.decorators.nodes if node.decorators else []
     for decorator in decorators:
         if (
-            isinstance(decorator, astroid.Attribute)
+            isinstance(decorator, nodes.Attribute)
             and decorator.attrname == "setter"
-            and isinstance(decorator.expr, astroid.Name)
+            and isinstance(decorator.expr, nodes.Name)
         ):
             return decorator.expr.name
     return None
@@ -64,9 +66,9 @@ def get_setters_property(node):
     """Get the property node for the given setter node.
 
     :param node: The node to get the property for.
-    :type node: astroid.FunctionDef
+    :type node: nodes.FunctionDef
 
-    :rtype: astroid.FunctionDef or None
+    :rtype: nodes.FunctionDef or None
     :returns: The node relating to the property of the given setter node,
         or None if one could not be found.
     """
@@ -99,13 +101,13 @@ def returns_something(return_node):
     if returns is None:
         return False
 
-    return not (isinstance(returns, astroid.Const) and returns.value is None)
+    return not (isinstance(returns, nodes.Const) and returns.value is None)
 
 
 def _get_raise_target(node):
-    if isinstance(node.exc, astroid.Call):
+    if isinstance(node.exc, nodes.Call):
         func = node.exc.func
-        if isinstance(func, (astroid.Name, astroid.Attribute)):
+        if isinstance(func, (nodes.Name, nodes.Attribute)):
             return utils.safe_infer(func)
     return None
 
@@ -125,19 +127,19 @@ def possible_exc_types(node):
 
 
     :param node: The raise node to find exception types for.
-    :type node: astroid.node_classes.NodeNG
+    :type node: nodes.NodeNG
 
     :returns: A list of exception types possibly raised by :param:`node`.
     :rtype: set(str)
     """
     excs = []
-    if isinstance(node.exc, astroid.Name):
+    if isinstance(node.exc, nodes.Name):
         inferred = utils.safe_infer(node.exc)
         if inferred:
             excs = [inferred.name]
     elif node.exc is None:
         handler = node.parent
-        while handler and not isinstance(handler, astroid.ExceptHandler):
+        while handler and not isinstance(handler, nodes.ExceptHandler):
             handler = handler.parent
 
         if handler and handler.type:
@@ -145,10 +147,10 @@ def possible_exc_types(node):
             excs = (exc.name for exc in inferred_excs if exc is not astroid.Uninferable)
     else:
         target = _get_raise_target(node)
-        if isinstance(target, astroid.ClassDef):
+        if isinstance(target, nodes.ClassDef):
             excs = [target.name]
-        elif isinstance(target, astroid.FunctionDef):
-            for ret in target.nodes_of_class(astroid.Return):
+        elif isinstance(target, nodes.FunctionDef):
+            for ret in target.nodes_of_class(nodes.Return):
                 if ret.frame() != target:
                     # return from inner function - ignore it
                     continue
@@ -156,7 +158,7 @@ def possible_exc_types(node):
                 val = utils.safe_infer(ret.value)
                 if (
                     val
-                    and isinstance(val, (astroid.Instance, astroid.ClassDef))
+                    and isinstance(val, (astroid.Instance, nodes.ClassDef))
                     and utils.inherit_from_std_ex(val)
                 ):
                     excs.append(val.name)
@@ -168,12 +170,12 @@ def possible_exc_types(node):
 
 
 def docstringify(docstring, default_type="default"):
-    for docstring_type in [
+    for docstring_type in (
         SphinxDocstring,
         EpytextDocstring,
         GoogleDocstring,
         NumpyDocstring,
-    ]:
+    ):
         instance = docstring_type(docstring)
         if instance.is_valid():
             return instance
@@ -202,6 +204,9 @@ class Docstring:
         doc = doc or ""
         self.doc = doc.expandtabs()
 
+    def __repr__(self) -> str:
+        return f"<{self.__class__.__name__}:'''{self.doc}'''>"
+
     def is_valid(self):
         return False
 
