@@ -1,16 +1,12 @@
-# -*- coding: utf-8 -*-
-
 # Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
-# For details: https://github.com/PyCQA/pylint/blob/master/COPYING
+# For details: https://github.com/PyCQA/pylint/blob/main/LICENSE
 
 """Looks for overlapping exceptions."""
 
 import astroid
 
-from pylint import interfaces
-from pylint import checkers
+from pylint import checkers, interfaces
 from pylint.checkers import utils
-
 from pylint.checkers.exceptions import _annotated_unpack_infer
 
 
