@@ -1,8 +1,8 @@
 # Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
-# For details: https://github.com/PyCQA/pylint/blob/master/LICENSE
+# For details: https://github.com/PyCQA/pylint/blob/main/LICENSE
 
 import sys
-from typing import List, Tuple
+from typing import List, Tuple, Union
 
 from pylint.constants import (
     _SCOPE_EXEMPT,
@@ -15,7 +15,11 @@ from pylint.constants import (
     MSG_TYPES_STATUS,
     WarningScope,
 )
-from pylint.exceptions import InvalidMessageError, UnknownMessageError
+from pylint.exceptions import (
+    InvalidMessageError,
+    NoLineSuppliedError,
+    UnknownMessageError,
+)
 from pylint.interfaces import UNDEFINED
 from pylint.message.message import Message
 from pylint.utils import get_module_and_frameid, get_rst_section, get_rst_title
@@ -59,6 +63,24 @@ class MessagesHandlerMixIn:
         )
         self._register_by_id_managed_msg(msgid, line)
 
+    def disable_next(
+        self,
+        msgid: str,
+        scope: str = "package",
+        line: Union[bool, int] = None,
+        ignore_unknown: bool = False,
+    ):
+        if not line:
+            raise NoLineSuppliedError
+        self._set_msg_status(
+            msgid,
+            enable=False,
+            scope=scope,
+            line=line + 1,
+            ignore_unknown=ignore_unknown,
+        )
+        self._register_by_id_managed_msg(msgid, line + 1)
+
     def enable(self, msgid, scope="package", line=None, ignore_unknown=False):
         self._set_msg_status(
             msgid, enable=True, scope=scope, line=line, ignore_unknown=ignore_unknown
@@ -274,6 +296,17 @@ class MessagesHandlerMixIn:
         # update stats
         msg_cat = MSG_TYPES[message_definition.msgid[0]]
         self.msg_status |= MSG_TYPES_STATUS[message_definition.msgid[0]]
+        if self.stats is None:
+            # pylint: disable=fixme
+            # TODO self.stats should make sense,
+            # class should make sense as soon as instantiated
+            # This is not true for Linter and Reporter at least
+            # pylint: enable=fixme
+            self.stats = {
+                msg_cat: 0,
+                "by_module": {self.current_name: {msg_cat: 0}},
+                "by_msg": {},
+            }
         self.stats[msg_cat] += 1
         self.stats["by_module"][self.current_name][msg_cat] += 1
         try:
@@ -291,7 +324,10 @@ class MessagesHandlerMixIn:
         else:
             module, obj = get_module_and_frameid(node)
             abspath = node.root().file
-        path = abspath.replace(self.reporter.path_strip_prefix, "", 1)
+        if abspath is not None:
+            path = abspath.replace(self.reporter.path_strip_prefix, "", 1)
+        else:
+            path = "configuration"
         # add the message
         self.reporter.handle_message(
             Message(
