@@ -1,7 +1,6 @@
 # Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
-# For details: https://github.com/PyCQA/pylint/blob/master/LICENSE
-
-from typing import List
+# For details: https://github.com/PyCQA/pylint/blob/main/LICENSE
+from typing import Dict, List, Optional, Tuple
 
 from pylint.exceptions import InvalidMessageError, UnknownMessageError
 
@@ -11,9 +10,9 @@ class MessageIdStore:
     """The MessageIdStore store MessageId and make sure that there is a 1-1 relation between msgid and symbol."""
 
     def __init__(self):
-        self.__msgid_to_symbol = {}
-        self.__symbol_to_msgid = {}
-        self.__old_names = {}
+        self.__msgid_to_symbol: Dict[str, str] = {}
+        self.__symbol_to_msgid: Dict[str, str] = {}
+        self.__old_names: Dict[str, List[str]] = {}
 
     def __len__(self):
         return len(self.__msgid_to_symbol)
@@ -39,14 +38,14 @@ class MessageIdStore:
             msg = f"'{symbol}' is not stored in the message store."
             raise UnknownMessageError(msg) from e
 
-    def register_message_definition(self, message_definition):
-        self.check_msgid_and_symbol(message_definition.msgid, message_definition.symbol)
-        self.add_msgid_and_symbol(message_definition.msgid, message_definition.symbol)
-        for old_msgid, old_symbol in message_definition.old_names:
+    def register_message_definition(
+        self, msgid: str, symbol: str, old_names: List[Tuple[str, str]]
+    ):
+        self.check_msgid_and_symbol(msgid, symbol)
+        self.add_msgid_and_symbol(msgid, symbol)
+        for old_msgid, old_symbol in old_names:
             self.check_msgid_and_symbol(old_msgid, old_symbol)
-            self.add_legacy_msgid_and_symbol(
-                old_msgid, old_symbol, message_definition.msgid
-            )
+            self.add_legacy_msgid_and_symbol(old_msgid, old_symbol, msgid)
 
     def add_msgid_and_symbol(self, msgid: str, symbol: str) -> None:
         """Add valid message id.
@@ -56,7 +55,9 @@ class MessageIdStore:
         self.__msgid_to_symbol[msgid] = symbol
         self.__symbol_to_msgid[symbol] = msgid
 
-    def add_legacy_msgid_and_symbol(self, msgid: str, symbol: str, new_msgid: str):
+    def add_legacy_msgid_and_symbol(
+        self, msgid: str, symbol: str, new_msgid: str
+    ) -> None:
         """Add valid legacy message id.
 
         There is a little duplication with add_msgid_and_symbol to avoid a function call,
@@ -68,24 +69,20 @@ class MessageIdStore:
         self.__old_names[msgid] = existing_old_names
 
     def check_msgid_and_symbol(self, msgid: str, symbol: str) -> None:
-        existing_msgid = self.__symbol_to_msgid.get(symbol)
-        existing_symbol = self.__msgid_to_symbol.get(msgid)
+        existing_msgid: Optional[str] = self.__symbol_to_msgid.get(symbol)
+        existing_symbol: Optional[str] = self.__msgid_to_symbol.get(msgid)
         if existing_symbol is None and existing_msgid is None:
-            return
+            return  # both symbol and msgid are usable
         if existing_msgid is not None:
             if existing_msgid != msgid:
                 self._raise_duplicate_msgid(symbol, msgid, existing_msgid)
         if existing_symbol != symbol:
-            self._raise_duplicate_symbol(msgid, symbol, existing_symbol)
+            # See https://github.com/python/mypy/issues/10559
+            self._raise_duplicate_symbol(msgid, symbol, existing_symbol)  # type: ignore
 
     @staticmethod
-    def _raise_duplicate_symbol(msgid, symbol, other_symbol):
-        """Raise an error when a symbol is duplicated.
-
-        :param str msgid: The msgid corresponding to the symbols
-        :param str symbol: Offending symbol
-        :param str other_symbol: Other offending symbol
-        :raises InvalidMessageError:"""
+    def _raise_duplicate_symbol(msgid: str, symbol: str, other_symbol: str):
+        """Raise an error when a symbol is duplicated."""
         symbols = [symbol, other_symbol]
         symbols.sort()
         error_message = f"Message id '{msgid}' cannot have both "
@@ -93,13 +90,8 @@ class MessageIdStore:
         raise InvalidMessageError(error_message)
 
     @staticmethod
-    def _raise_duplicate_msgid(symbol, msgid, other_msgid):
-        """Raise an error when a msgid is duplicated.
-
-        :param str symbol: The symbol corresponding to the msgids
-        :param str msgid: Offending msgid
-        :param str other_msgid: Other offending msgid
-        :raises InvalidMessageError:"""
+    def _raise_duplicate_msgid(symbol: str, msgid: str, other_msgid: str) -> None:
+        """Raise an error when a msgid is duplicated."""
         msgids = [msgid, other_msgid]
         msgids.sort()
         error_message = (
@@ -112,14 +104,14 @@ class MessageIdStore:
     def get_active_msgids(self, msgid_or_symbol: str) -> List[str]:
         """Return msgids but the input can be a symbol."""
         # Only msgid can have a digit as second letter
-        is_msgid = msgid_or_symbol[1:].isdigit()
+        is_msgid: bool = msgid_or_symbol[1:].isdigit()
         if is_msgid:
             msgid = msgid_or_symbol.upper()
             symbol = self.__msgid_to_symbol.get(msgid)
         else:
-            msgid = self.__symbol_to_msgid.get(msgid_or_symbol)
+            msgid = self.__symbol_to_msgid.get(msgid_or_symbol)  # type: ignore
             symbol = msgid_or_symbol
-        if not msgid or not symbol:
+        if msgid is None or symbol is None or not msgid or not symbol:
             error_msg = f"No such message id or symbol '{msgid_or_symbol}'."
             raise UnknownMessageError(error_msg)
         return self.__old_names.get(msgid, [msgid])
