@@ -1,8 +1,5 @@
-# -*- coding: utf-8 -*-
-
 # Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
-# For details: https://github.com/PyCQA/pylint/blob/master/COPYING
-
+# For details: https://github.com/PyCQA/pylint/blob/main/LICENSE
 from pylint.reporters.base_reporter import BaseReporter
 
 
@@ -15,7 +12,7 @@ class CollectingReporter(BaseReporter):
         BaseReporter.__init__(self)
         self.messages = []
 
-    def handle_message(self, msg):
-        self.messages.append(msg)
+    def reset(self):
+        self.messages = []
 
     _display = None
