@@ -3,9 +3,11 @@
 # Copyright (c) 2019-2021 Pierre Sassoulas <pierre.sassoulas@gmail.com>
 # Copyright (c) 2019 Hugo van Kemenade <hugovk@users.noreply.github.com>
 # Copyright (c) 2020 hippo91 <guillaume.peillex@gmail.com>
+# Copyright (c) 2021 Marc Mueller <30130371+cdce8p@users.noreply.github.com>
+# Copyright (c) 2021 bot <bot@noreply.github.com>
 
 # Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
-# For details: https://github.com/PyCQA/pylint/blob/master/LICENSE
+# For details: https://github.com/PyCQA/pylint/blob/main/LICENSE
 
 """Text formatting drivers for ureports"""
 
@@ -69,8 +71,8 @@ class TextWriter(BaseWriter):
         format_strings = " ".join(["%%-%ss"] * len(cols_width))
         format_strings = format_strings % tuple(cols_width)
         format_strings = format_strings.split(" ")
-        table_linesep = "\n+" + "+".join(["-" * w for w in cols_width]) + "+\n"
-        headsep = "\n+" + "+".join(["=" * w for w in cols_width]) + "+\n"
+        table_linesep = "\n+" + "+".join("-" * w for w in cols_width) + "+\n"
+        headsep = "\n+" + "+".join("=" * w for w in cols_width) + "+\n"
 
         self.write(table_linesep)
         for index, line in enumerate(table_content):
