@@ -1,10 +1,12 @@
 # Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
-# For details: https://github.com/PyCQA/pylint/blob/master/LICENSE
+# For details: https://github.com/PyCQA/pylint/blob/main/LICENSE
 
 from io import StringIO
 from os import getcwd, linesep, sep
+from typing import Dict, List
 
 from pylint import interfaces
+from pylint.message import Message
 from pylint.reporters import BaseReporter
 
 
@@ -14,21 +16,20 @@ class GenericTestReporter(BaseReporter):
     __implements__ = interfaces.IReporter
 
     def __init__(self):  # pylint: disable=super-init-not-called
-
-        self.message_ids = {}
         self.reset()
-        self.path_strip_prefix = getcwd() + sep
 
     def reset(self):
+        self.message_ids: Dict = {}
         self.out = StringIO()
-        self.messages = []
+        self.path_strip_prefix: str = getcwd() + sep
+        self.messages: List[str] = []
 
-    def handle_message(self, msg):
+    def handle_message(self, msg: Message) -> None:
         """manage message of different type and in the context of path"""
         obj = msg.obj
         line = msg.line
         msg_id = msg.msg_id
-        msg = msg.msg
+        str_message: str = msg.msg
         self.message_ids[msg_id] = 1
         if obj:
             obj = ":%s" % obj
@@ -36,8 +37,8 @@ class GenericTestReporter(BaseReporter):
         if linesep != "\n":
             # 2to3 writes os.linesep instead of using
             # the previously used line separators
-            msg = msg.replace("\r\n", "\n")
-        self.messages.append(f"{sigle}:{line:>3}{obj}: {msg}")
+            str_message = str_message.replace("\r\n", "\n")
+        self.messages.append(f"{sigle}:{line:>3}{obj}: {str_message}")
 
     def finalize(self):
         self.messages.sort()
@@ -60,21 +61,18 @@ class GenericTestReporter(BaseReporter):
 
 
 class MinimalTestReporter(BaseReporter):
-    def handle_message(self, msg):
-        self.messages.append(msg)
-
     def on_set_current_module(self, module, filepath):
         self.messages = []
 
     _display = None
 
 
-class FunctionalTestReporter(BaseReporter):  # pylint: disable=abstract-method
-    def handle_message(self, msg):
-        self.messages.append(msg)
-
+class FunctionalTestReporter(BaseReporter):
     def on_set_current_module(self, module, filepath):
         self.messages = []
 
     def display_reports(self, layout):
         """Ignore layouts and don't call self._display()."""
+
+    def _display(self, layout):
+        pass
