@@ -1,5 +1,6 @@
 # Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
-# For details: https://github.com/PyCQA/pylint/blob/master/COPYING
+# For details: https://github.com/PyCQA/pylint/blob/main/LICENSE
+
 
 try:
     import isort.api
@@ -33,6 +34,23 @@ def normalize_text(text, line_len=DEFAULT_LINE_LENGTH, indent=""):
     )
 
 
+CMPS = ["=", "-", "+"]
+
+
+# py3k has no more cmp builtin
+def cmp(a, b):  # pylint: disable=redefined-builtin
+    return (a > b) - (a < b)
+
+
+def diff_string(old, new):
+    """given an old and new int value, return a string representing the
+    difference
+    """
+    diff = abs(old - new)
+    diff_str = "{}{}".format(CMPS[cmp(old, new)], diff and ("%.2f" % diff) or "")
+    return diff_str
+
+
 def get_module_and_frameid(node):
     """return the module name and the frame id in the module"""
     frame = node.frame()
@@ -45,14 +63,14 @@ def get_module_and_frameid(node):
         try:
             frame = frame.parent.frame()
         except AttributeError:
-            frame = None
+            break
     obj.reverse()
     return module, ".".join(obj)
 
 
 def get_rst_title(title, character):
     """Permit to get a title formatted as ReStructuredText test (underlined with a chosen character)."""
-    return "%s\n%s\n" % (title, character * len(title))
+    return f"{title}\n{character * len(title)}\n"
 
 
 def get_rst_section(section, options, doc=None):
@@ -97,145 +115,6 @@ def tokenize_module(module):
         return list(tokenize.tokenize(readline))
 
 
-def _basename_in_blacklist_re(base_name, black_list_re):
-    """Determines if the basename is matched in a regex blacklist
-
-    :param str base_name: The basename of the file
-    :param list black_list_re: A collection of regex patterns to match against.
-        Successful matches are blacklisted.
-
-    :returns: `True` if the basename is blacklisted, `False` otherwise.
-    :rtype: bool
-    """
-    for file_pattern in black_list_re:
-        if file_pattern.match(base_name):
-            return True
-    return False
-
-
-def _modpath_from_file(filename, is_namespace, path=None):
-    def _is_package_cb(path, parts):
-        return modutils.check_modpath_has_init(path, parts) or is_namespace
-
-    return modutils.modpath_from_file_with_callback(
-        filename, path=path, is_package_cb=_is_package_cb
-    )
-
-
-def get_python_path(filepath):
-    dirname = os.path.realpath(os.path.expanduser(filepath))
-    if not os.path.isdir(dirname):
-        dirname = os.path.dirname(dirname)
-    while True:
-        if not os.path.exists(os.path.join(dirname, "__init__.py")):
-            return dirname
-        old_dirname = dirname
-        dirname = os.path.dirname(dirname)
-        if old_dirname == dirname:
-            return os.getcwd()
-    return None
-
-
-def expand_modules(files_or_modules, black_list, black_list_re):
-    """take a list of files/modules/packages and return the list of tuple
-    (file, module name) which have to be actually checked
-    """
-    result = []
-    errors = []
-    path = sys.path.copy()
-
-    for something in files_or_modules:
-        if os.path.basename(something) in black_list:
-            continue
-        if _basename_in_blacklist_re(os.path.basename(something), black_list_re):
-            continue
-
-        module_path = get_python_path(something)
-        additional_search_path = [".", module_path] + path
-        if os.path.exists(something):
-            # this is a file or a directory
-            try:
-                modname = ".".join(
-                    modutils.modpath_from_file(something, path=additional_search_path)
-                )
-            except ImportError:
-                modname = os.path.splitext(os.path.basename(something))[0]
-            if os.path.isdir(something):
-                filepath = os.path.join(something, "__init__.py")
-            else:
-                filepath = something
-        else:
-            # suppose it's a module or package
-            modname = something
-            try:
-                filepath = modutils.file_from_modpath(
-                    modname.split("."), path=additional_search_path
-                )
-                if filepath is None:
-                    continue
-            except (ImportError, SyntaxError) as ex:
-                # The SyntaxError is a Python bug and should be
-                # removed once we move away from imp.find_module: http://bugs.python.org/issue10588
-                errors.append({"key": "fatal", "mod": modname, "ex": ex})
-                continue
-
-        filepath = os.path.normpath(filepath)
-        modparts = (modname or something).split(".")
-
-        try:
-            spec = modutils.file_info_from_modpath(
-                modparts, path=additional_search_path
-            )
-        except ImportError:
-            # Might not be acceptable, don't crash.
-            is_namespace = False
-            is_directory = os.path.isdir(something)
-        else:
-            is_namespace = modutils.is_namespace(spec)
-            is_directory = modutils.is_directory(spec)
-
-        if not is_namespace:
-            result.append(
-                {
-                    "path": filepath,
-                    "name": modname,
-                    "isarg": True,
-                    "basepath": filepath,
-                    "basename": modname,
-                }
-            )
-
-        has_init = (
-            not (modname.endswith(".__init__") or modname == "__init__")
-            and os.path.basename(filepath) == "__init__.py"
-        )
-        if has_init or is_namespace or is_directory:
-            for subfilepath in modutils.get_module_files(
-                os.path.dirname(filepath), black_list, list_all=is_namespace
-            ):
-                if filepath == subfilepath:
-                    continue
-                if _basename_in_blacklist_re(
-                    os.path.basename(subfilepath), black_list_re
-                ):
-                    continue
-
-                modpath = _modpath_from_file(
-                    subfilepath, is_namespace, path=additional_search_path
-                )
-                submodname = ".".join(modpath)
-                result.append(
-                    {
-                        "path": subfilepath,
-                        "name": submodname,
-                        "isarg": False,
-                        "basepath": filepath,
-                        "basename": modname,
-                    }
-                )
-    return result, errors
-
-
 def register_plugins(linter, directory):
     """load all module and package in the given directory, looking for a
     'register' function in each one, used to register pylint checkers
@@ -248,7 +127,11 @@ def register_plugins(linter, directory):
         if (
             extension in PY_EXTS
             and base != "__init__"
-            or (not extension and os.path.isdir(os.path.join(directory, base)))
+            or (
+                not extension
+                and os.path.isdir(os.path.join(directory, base))
+                and not filename.startswith(".")
+            )
         ):
             try:
                 module = modutils.load_module_from_file(
@@ -258,9 +141,7 @@ def register_plugins(linter, directory):
                 # empty module name (usually emacs auto-save files)
                 continue
             except ImportError as exc:
-                print(
-                    "Problem importing module %s: %s" % (filename, exc), file=sys.stderr
-                )
+                print(f"Problem importing module {filename}: {exc}", file=sys.stderr)
             else:
                 if hasattr(module, "register"):
                     module.register(linter)
@@ -268,7 +149,7 @@ def register_plugins(linter, directory):
 
 
 def get_global_option(checker, option, default=None):
-    """ Retrieve an option defined by the given *checker* or
+    """Retrieve an option defined by the given *checker* or
     by all known option providers.
 
     It will look in the list of all options providers
@@ -364,10 +245,12 @@ def _comment(string):
 
 def _format_option_value(optdict, value):
     """return the user input's value from a 'compiled' value"""
-    if isinstance(value, (list, tuple)):
+    if optdict.get("type", None) == "py_version":
+        value = ".".join(str(item) for item in value)
+    elif isinstance(value, (list, tuple)):
         value = ",".join(_format_option_value(optdict, item) for item in value)
     elif isinstance(value, dict):
-        value = ",".join("%s:%s" % (k, v) for k, v in value.items())
+        value = ",".join(f"{k}:{v}" for k, v in value.items())
     elif hasattr(value, "match"):  # optdict.get('type') == 'regexp'
         # compiled regexp
         value = value.pattern
@@ -406,23 +289,27 @@ def _ini_format(stream, options):
                 value = separator.join(x + "," for x in str(value).split(","))
                 # remove trailing ',' from last element of the list
                 value = value[:-1]
-            print("%s=%s" % (optname, value), file=stream)
+            print(f"{optname}={value}", file=stream)
 
 
 class IsortDriver:
     """A wrapper around isort API that changed between versions 4 and 5."""
 
-    def __init__(self, known_third_party, known_standard_library):
+    def __init__(self, config):
         if HAS_ISORT_5:
             self.isort5_config = isort.api.Config(
-                known_third_party=known_third_party,
-                known_standard_library=known_standard_library,
+                # There is not typo here. EXTRA_standard_library is
+                # what most users want. The option has been named
+                # KNOWN_standard_library for ages in pylint and we
+                # don't want to break compatibility.
+                extra_standard_library=config.known_standard_library,
+                known_third_party=config.known_third_party,
             )
         else:
             self.isort4_obj = isort.SortImports(  # pylint: disable=no-member
                 file_contents="",
-                known_third_party=known_third_party,
-                known_standard_library=known_standard_library,
+                known_standard_library=config.known_standard_library,
+                known_third_party=config.known_third_party,
             )
 
     def place_module(self, package):
